@@ -432,12 +432,15 @@ library if you like.
 
     Tone levels are determined as follows:
     * Tone levels are set in this method if not set by
-      ((<set_tone_levels>)) or both the options two (({"levels"})) and
-      (({"patterns"})) are specified explicitly.
+      ((<set_tone_levels>)) or the option (({"levels"})) (and
+      optionally, (({"patterns"}))) is (are) specified explicitly.
     * When contour levels & patterns are set in this method, 
-       * The combination of (({"levels"})) and (({"patterns"})) 
-         have the highest precedence. If they are specified,
+       * (({"levels"})) has the highest precedence. If it is specified,
          tone levels and patterns are determined by DCLExt::((<ue_set_tone>)).
+         Here, tone patterns can be specified with the option (({"patterns"})).
+       * Currently, option (({"patterns"})) is effective only if (({"levels"}))
+         is specified. Otherwise, it is ignored and patterns are
+         determined internally (by using DCL.uegtlb).
        * If not, a linear tone levels are set if (({"ltone"=true}))
          (this is the default), or shading is made if (({"ltone"=false})).
          Shading is determined by following the parameters in the UDPACK
@@ -466,6 +469,12 @@ library if you like.
        "annotate"    true    # if false, do not put texts on the right
                              # margin even when newframe==true
        "ltone"       true    # Same as udpack parameter ltone
+       "clr_min"     nil     # if an integer (in 10..99) is specified, used as
+			     # the mimimum color number. (the same can be done
+			     # by setting the uepack parameter "icolor1")
+       "clr_max"     nil     # if an integer (in 10..99) is specified, used as
+			     # the maximum color number. (the same can be done
+			     # by setting the uepack parameter "icolor2")
        "min"         nil     # minimum contour value
        "max"         nil     # maximum contour value
        "nlev"        nil     # number of levels
@@ -1167,15 +1176,22 @@ module NumRu
         DCL.uscset('cxunit',(opts['xunits'] || xai['units']) )
         DCL.uspset('dxt', opts['xtickint'])  if(opts['xtickint'])
         DCL.uspset('dxl', opts['xlabelint']) if(opts['xlabelint'])
-        opts['xside'].split('').each{|s| DCL.usxaxs(s)}  # scan('.') also works
+        opts['xside'].split('').each{|s|     # scan('.') also works
+	  DCL.usxaxs(s)
+	}
       end
       if opts['yside'].length > 0
         yai = _get_axinfo(yax)
         DCL.uscset('cyttl', (opts['ytitle'] || yai['title']) )
-        DCL.uscset('cyunit',(opts['yunits'] || yai['units']) )
+        DCL.uscset('cyunit', (un=(opts['yunits'] || yai['units'])) )
         DCL.uspset('dxt', opts['ytickint'])  if(opts['ytickint'])
         DCL.uspset('dxl', opts['ylabelint']) if(opts['ylabelint'])
-        opts['yside'].split('').each{|s| DCL.usyaxs(s)}  # scan('.') also works
+        opts['yside'].split('').each{|s|   # scan('.') also works
+	  DCL.usyaxs(s)
+	  if s=='l' && un && un != ''
+	    DCL.uzpset('roffxt', DCL.uzpget('rsizec1')*1.5 )
+	  end
+	}
       end
       nil
     end
@@ -1206,7 +1222,8 @@ module NumRu
           ['annotate', true, 'if false, do not put texts on the right margin even when newframe==true'],
           ['exchange', false, 'whether to exchange x and y axes'],
           ['index', 1, 'line/mark index'],
-          ['type', 1, 'line type']
+          ['type', 1, 'line type'],
+          ['label', nil, 'if a String is given, it is shown as the label']
         )
       end
       opts = @@line_options.interpret(options)
@@ -1224,7 +1241,13 @@ module NumRu
         title( (opts['title'] || gp.data.get_att('long_name')) )
         annotate(gp.lost_axes) if opts['annotate']
       end
+      if opts['label']
+	lcharbk = DCL.sgpget('lchar')
+	DCL.sgpset('lchar',true)
+	DCL.sgsplc(opts['label'])
+      end
       DCL.uulinz(x.val, y.val, opts['type'], opts['index'])
+      DCL.sgpset('lchar',lcharbk) if opts['label']
       nil
     end
 
@@ -1318,6 +1341,7 @@ module NumRu
         @@contour_options = Misc::KeywordOptAutoHelp.new(
           ['title', nil, 'Title of the figure(if nil, internally determined)'],
           ['annotate', true, 'if false, do not put texts on the right margin even when newframe==true'],
+          ['transpose', false, 'if true, exchange x and y axes'],
           @@linear_contour_options,
           @@contour_levels
         )
@@ -1329,6 +1353,7 @@ module NumRu
       end
       opts = @@contour_options.interpret(options)
       gp = gphys.first2D
+      gp = gp.transpose(1,0) if opts['transpose']
       xax = gp.coord(0)
       yax = gp.coord(1)
       if newframe
@@ -1379,9 +1404,25 @@ module NumRu
       _set_tone_levels_(opts)
     end
     def _set_tone_levels_(opts)
-      raise ArgumentError, "'levels' must be explicitly set" if !opts['levels']
-      raise ArgumentError, "'patternss' must be explicitly set" if !opts['patterns']
-      DCLExt.ue_set_tone(opts['levels'],opts['patterns'])
+      if !opts['levels'] && !opts['patterns']
+      end
+      if opts['levels']
+	levels = opts['levels']
+      else
+	raise ArgumentError,"'levels' must be explicitly set"
+      end
+      if opts['patterns']
+	patterns = opts['patterns']
+      else
+	nlev = levels.length - 1
+	itpat = DCL.ueiget('itpat')
+	iclr1 = DCL.ueiget('icolor1')
+	iclr2 = DCL.ueiget('icolor2')
+	patterns = (0...nlev).collect{|i|
+	  (((iclr2-iclr1)/(nlev-1.0)*i+iclr1).round) * 1000 + itpat
+	}
+      end
+      DCLExt.ue_set_tone(levels,patterns)
       @@set_tone_levels=true
       nil
     end
@@ -1399,13 +1440,25 @@ module NumRu
           ['title', nil, 'Title of the figure(if nil, internally determined)'],
           ['annotate', true, 'if false, do not put texts on the right margin even when newframe==true'],
           ['ltone', true, 'Same as udpack parameter ltone'],
+          ['clr_min', nil,  'if an integer (in 10..99) is specified, used as the mimimum color number. (the same can be done by setting the uepack parameter "icolor1")'],
+          ['clr_max', nil,  'if an integer (in 10..99) is specified, used as the maximum color number. (the same can be done by setting the uepack parameter "icolor2")'],
+          ['transpose', false, 'if true, exchange x and y axes'],
           @@linear_tone_options,
           @@tone_levels
         )
       end
       opts = @@tone_options.interpret(options)
 
+      if opts['clr_min']
+	icolor1_bak = DCL.uepget('icolor1')
+	DCL.uepset('icolor1', opts['clr_min'])
+      end
+      if opts['clr_max']
+	icolor2_bak = DCL.uepget('icolor2')
+	DCL.uepset('icolor2', opts['clr_max'])
+      end
       gp = gphys.first2D
+      gp = gp.transpose(1,0) if opts['transpose']
       xax = gp.coord(0)
       yax = gp.coord(1)
       if newframe
@@ -1413,11 +1466,13 @@ module NumRu
       end
       DCL.uwsgxa(xax.val)
       DCL.uwsgya(yax.val)
-      if opts['levels'] && opts['patterns']
+      if opts['levels'] 
         backup = @@set_tone_levels
         _set_tone_levels_(opts)
         @@set_tone_levels = backup
         DCL.uetone(gp.data.val)
+      elsif opts['patterns']
+	raise "option 'patterns' is not effective unless 'levels' is specified"
       elsif !@@set_tone_levels && opts['ltone']
         DCLExt.ue_set_linear_levs(gp.data.val, opts)
         DCL.uetone(gp.data.val)
@@ -1430,6 +1485,8 @@ module NumRu
         title( (opts['title'] || gp.data.get_att('long_name')) )
         annotate(gp.lost_axes) if opts['annotate']
       end
+      DCL.uepset('icolor1', icolor1_bak) if opts['clr_min']
+      DCL.uepset('icolor2', icolor2_bak) if opts['clr_max']
       nil
     end
 
@@ -1542,6 +1599,11 @@ if $0 == __FILE__
   GGraph.tone(gphys, true, 'levels'=>[-10,10],'patterns'=>[50999,80999])
   #/ graph 16 /
   GGraph.tone(gphys, true, 'levels'=>[-10,10],'patterns'=>[50999])
+  #/ graph 17 /
+  GGraph.set_fig('viewport'=>[0.3,0.7,0.1,0.6])
+  GGraph.contour(gphys,true, 'min'=>0, 'coloring'=>true, 'transpose'=>true)
+  GGraph.tone(gphys, false, 'levels'=>[10,20], 'patterns'=>[80999], 'transpose'=>true)
+
   DCL.grcls
 
 end
