@@ -87,7 +87,8 @@ module NumRu
       end
 
       def [](*slicer)
-	 if slicer.length==1 && slicer[0].is_a?(Hash)
+	 if slicer.length==1 && slicer[0].is_a?(Hash) && 
+	    slicer[0].keys[0].is_a?(String)
 	   slicer = __process_hash_slicer(slicer[0])
 	 else
 	   slicer = __rubber_expansion( slicer )
@@ -98,7 +99,8 @@ module NumRu
       def []=(*args)
 	 val = args.pop
 	 slicer = args
-	 if slicer.length==1 && slicer[0].is_a?(Hash)
+	 if slicer.length==1 && slicer[0].is_a?(Hash) && 
+	    slicer[0].keys[0].is_a?(String)
 	   slicer = __process_hash_slicer(slicer[0])
 	 else
 	   slicer = __rubber_expansion( slicer )
@@ -217,6 +219,12 @@ module NumRu
 	 end
       end
 
+      def transpose(*dims)
+	grid = @grid.transpose(*dims)
+	data = @data.transpose(*dims)
+	GPhys.new( grid, data )
+      end
+
       for f in VArray::Math_funcs
 	 eval <<-EOS, nil, __FILE__, __LINE__+1
 	 #def GPhys.#{f}(gphys)
@@ -367,7 +375,7 @@ module NumRu
                                     # rtable will be merged in the following
 
 	    shs.each_index do |i|
-	       print axnms[i]," ",axnml[ table[i] ]," ",axnml[ rtable[i] ],"\n"
+	       #print axnms[i]," ",axnml[ table[i] ]," ",axnml[ rtable[i] ],"\n"
 	       if axnms[i] == axnml[ rtable[i] ]
 		  real_table[i] = rtable[i]
 	       elsif  axnms[i] != axnml[ table[i] ]
@@ -451,4 +459,18 @@ if $0 == __FILE__
    p gpz.val, gpz_m0.val, gpz_m0.lost_axes
    p gpz.mean, gpz.max
    p gpz.mean("x").val
+
+   print "transpose\n"
+   p gpz3.axnames, gpz3.val, 
+     gpz3.transpose(2,0,1).axnames, gpz3.transpose(2,0,1).val
+
+   print "manual cyclic extention\n"
+   p(sp=gpz3.shape)
+   gpz3_cext = gpz3[ [0...sp[0],0], false ]
+   p gpz3_cext.coord(0).val, gpz3_cext.val
+
+   print "axis to gphys\n"
+   ax = gpz3.axis(1)
+   p ax.to_gphys
+   p ax.to_gphys( ax.cell_bounds[0..-2].copy )
 end
