@@ -30,7 +30,7 @@ helps read/write GrADS-formatted data
 
 ---write(file, gphys, name=nil)
 
-    writes a GPhys object into a GrADS file. -- !!DOES NOT WORK CURRENTLY!!
+    writes a GPhys object into a GrADS file. -- !!only 4D data is supported!!
 
 =end
 
@@ -95,28 +95,13 @@ module NumRu
       end
 
       def write(file, gphys, name=nil)
-        (0...(gphys.rank)).each{|i|
-          ax = gphys.axis(i)
-          dimname = ax.pos.name
-          length = ax.pos.length
-          isfx = 0
-          altdimnames = Hash.new
-          ax.flatten.each{ |va|
-            if va.length == length
-              dimnames = [dimname]
-            else
-              if (nm=altdimnames[va.length])
-                dimnames = [nm]
-              else
-                dimnames = [ (altdimnames[va.length] = dimname+isfx.to_s) ]
-                isfx += 1
-              end
-            end
-            p va.type
-            VArrayGrADS.write(file, va, nil, dimnames )
-          }
-        }
-        VArrayGrADS.write(file, gphys.data, name, gphys.axnames)
+        if file.is_a?(String)
+          file = GrADS_Gridded.open(file,"w")
+        elsif ! file.is_a?(GrADS_Gridded)
+          raise ArgumentError, "1st arg must be a GrADS_Gridded or a file name"
+        end
+        VArrayGrADS.write_control(file, gphys)
+        VArrayGrADS.write_binary(file, gphys.data)
       end
 
     end
@@ -159,4 +144,12 @@ if $0 == __FILE__
    GPhys::NetCDF_IO.write(file2,temp_xmean)
    file3.close
 
+   puts "\n** test write (tmp.ctl) **"
+   file1 = GrADS_Gridded.create("tmp2.ctl")
+   file1.put_att("big_endian",true)
+   file1.put_att("dset","tmp2.dr")
+   file1.put_att("title","test control file")
+   file1.put_att("undef",-999.0)
+   GPhys::GrADS_IO.write(file1,temp_edy)
+
 end
