@@ -50,12 +50,34 @@ through which the users can also access its binary data file(s).
      does not work. 
 
 ==Methods
+---ndims
+   returns the number of dimensions in the file (always 4). 
+
 ---nvars
    returns the number of variables in the file. 
 
+---natts
+   returns the number of attributes of the variable.
+
 ---path
    returns the path of the control file.
 
+---put_att( attname, value )
+   set global attribute
+
+   ARGUMENTS
+   * attrname (String): name of an attribute
+   * value (String): value of an attribute
+
+---def_var(name="noname",nlev=0,option="99",description="")
+   define a variable
+
+   ARGUMENTS
+   * name (String): name of the variable
+   * nlev (Integer): number of vertical levels
+   * option (String): variable placement option 
+     ("99": normal, "-1,30": transpose lon and lat)
+
 ---var( varname=nil )
    opens an existing variable in the file. 
 
@@ -75,12 +97,17 @@ through which the users can also access its binary data file(s).
    * Array of GrADSVar objects. 
      returns all variables if names==nil
 
+---get_att( key=nil )
+   returns tha value of the global attribute
+
 ---dim_names
    returns the names of all dimensions in the control file.
 
 ---var_names
    returns the names of all variables in the control file.
 
+---att_names
+   returns the names of all the global attributes.
 
 ---to_ctl
    returns the contents of the corresponding control file as a String.
@@ -103,6 +130,12 @@ through which the users can also access its binary data file(s).
      from 0). Currently only one time must be chosen, but in the
      future, it is planned to support multiple times.
 
+---put(ary)
+   writes the NArray on the binary data file. 
+
+   ARGUMENTS
+   * ary (NArray): data to write. 
+
 ---varnames
    Returns names of the variable in the GrADS file as an Array in the order
    placed.
@@ -132,9 +165,6 @@ through which the users can also access its binary data file(s).
      has no effect (the former is a substitution IN a Hash, while the latter
      is a substitution OF the Hash).
 
----ctlfilename
-   path of the control file
-
 ---get_dim(dim)
    returns positions of a dimension as an NArray.
 
@@ -186,15 +216,6 @@ It is intended to behave as a correspondent of a NetCDFVar object.
 
 ==Methods
 
----name
-   returns the name of the variable. 
-
----vartype
-   returns "sfloat" in order to behave as NetCDFVar#vartype. 
-
----dim_names
-   returns the names of all dimensions in the control file.
-
 ---shape_ul0
    returns the shape of the variable, but the length of the unlimited 
    dimension is set to zero.
@@ -208,6 +229,40 @@ It is intended to behave as a correspondent of a NetCDFVar object.
    RETURN VALUE
    *  Array. [length of 0th dim, length of 1st dim,.. ]
 
+---dim_names
+   returns the names of all dimensions of the variable.
+
+---att_names
+   returns the names of all attributes of the variable.
+
+---name
+   returns the name of the variable. 
+
+---ndims
+   returns the number of dimensions in the file (always 4). 
+
+---rank
+   alias of ndims
+
+---vartype
+   returns "sfloat" in order to behave as NetCDFVar#vartype. 
+
+---natts
+   returns the number of attributes of the variable.
+
+---file
+   returns the file name that controls the variable. 
+
+---get_att( name=nil )
+   returns tha value of the attribute of the variable.
+
+---put_att( name, value )
+   set an attribute of the variable. 
+
+   ARGUMENTS
+   * name (String): name of an attribute
+   * value (String): value of an attribute
+
 ---get(hash=nil)
    returns values of the variable.
 
@@ -254,10 +309,16 @@ class GrADS_Gridded
         raise "share = true is not supported."
       end
       if (File.exists?(ctlfilename))
-        raise "#{ctlfilename} already exists."
+        print "#{ctlfilename} already exists.\n"
+        print "overwrite #{ctlfilename} (y/n)? "
+        ans = gets[0].chr
+        if ans != "y"
+          raise "#{ctlfilename} already exists."
+        end
       end
 
-      GrADS_Gridded.new(ctlfilename, "w+")
+#      GrADS_Gridded.new(ctlfilename, "w+")
+      GrADS_Gridded.new(ctlfilename, "w")
     end
   end
 
@@ -267,7 +328,7 @@ class GrADS_Gridded
     when /^r/
       @mode = 'r'
     when /^w/
-      @mode = 'w+'
+      @mode = 'w'
     else
       raise ArgumentError, "Unsupported IO mode: #{mode}"
     end
@@ -284,15 +345,15 @@ class GrADS_Gridded
       "cray_32bit_ieee"=>nil,
     }
 
-    if (File.exists?(ctlfilename))
-      @ctlfile = File.open(ctlfilename, mode)
-      parse_ctl
-    else  # create new file
-#      if @mode != 'w'
-      if @mode != 'w+'
+    case(@mode)
+    when('r')
+      if (File.exists?(ctlfilename))
+        @ctlfile = File.open(ctlfilename, mode)
+        parse_ctl
+      else
         raise "File #{ctlfilename} does not exist."
       end
-#      initialize_new
+    when('w')
       @ctlfile = File.open(ctlfilename, mode)
       
       @dimensions = []
@@ -308,13 +369,21 @@ class GrADS_Gridded
       #<internal control parameters>
       @define_mode = true
       @ctl_dumped = false
-
+    else
+      raise ArgumentError, "Unsupported IO mode: #{@mode}"
     end
   end
 
-
-
   def close
+    @ctlfile.close
+  end
+
+  def path
+    @ctlfile.path
+  end
+
+  def ndims
+    4
   end
 
   def nvars
@@ -322,33 +391,41 @@ class GrADS_Gridded
   end
 
   def natts
+    2
   end
 
-  def unlimited
-  end
-
-  def path
-    @ctlfile.path
-  end
-
+#  def unlimited
 #  def redef
 #  def enddef
 #  def define_mode?
 #  def sync
+#  def def_dim(ary)
 
-  def def_dim(ary)
-
+  def put_att(key,value)
+    if(key == "dset")
+      @dset = value
+    elsif(key == "title")
+      @title = value
+    elsif(key == "undef")
+      @undef = value
+    else
+      if ! (value.is_a?(TrueClass) || value.is_a?(NilClass) )
+        raise ArgumentError, "2nd arg: not a true nor nil" 
+      end
+      if (@options.has_key?(key))
+        @options[key] = value
+      else
+        raise "Invalid/unsupported option: "+key
+      end
+    end
   end
 
-  def put_att
+  def def_var(name="noname",nlev=0,option="99",description="")
+    @variables.push({:name=>name.to_s, :nlev=>nlev.to_s,
+                     :option=>option.to_s, :description=>description.to_s})
   end
 
-  def def_var(name)
-    @variables.push({:name=>name})
-  end
-
-  def def_var_with_dim
-  end
+#  def def_var_with_dim
 
   def var( varname=nil )
     GrADSVar.new(self,varname)
@@ -365,26 +442,26 @@ class GrADS_Gridded
     vars
   end
 
-  def dim
+#  def dim
+#  def dims
+
+#  def att
+  def get_att( key=nil )
+    case(key)
+    when("dset")
+      att = @dset
+    when("title")
+      att = @title
+    else
+      raise "Invalid/unsupported option: "+key
+    end
+    att
   end
 
-  def dims
-  end
-
-  def att
-  end
-
-  def fill=
-  end
-
-  def each_dim
-  end
-
-  def each_var
-  end
-
-  def each_att
-  end
+#  def fill=
+#  def each_dim
+#  def each_var
+#  def each_att
 
   def dim_names
     ary = Array.new()
@@ -399,22 +476,23 @@ class GrADS_Gridded
   end
 
   def att_names
+    ary = ["dset","title"]
+    ary
   end
 
-
   def to_ctl
     return <<EOS
 DSET    #{@dset}
 TITLE   #{@title}
 UNDEF   #{@undef}
 OPTIONS #{op=""; @options.each{|key,val| op += key+" " if(val)}; op}
-XDEF    #{@dimensions[0][:len]}\t#{@dimensions[0][:flag]}\t#{@dimensions[0][:spec]}
-YDEF    #{@dimensions[1][:len]}\t#{@dimensions[1][:flag]}\t#{@dimensions[1][:spec]}
-ZDEF    #{@dimensions[2][:len]}\t#{@dimensions[2][:flag]}\t#{@dimensions[2][:spec]}
-TDEF    #{@dimensions[3][:len]}\t#{@dimensions[3][:flag]}\t#{@dimensions[3][:spec]}
-VARS
-#{@variables.collect{|i| "  "+i[:name]+"\t"+i[:nlev].to_s+"\t"+i[:option].to_s+
-       "\t"+i[:description]}.join("\n")}
+XDEF    #{@dimensions[0][:len]} #{@dimensions[0][:flag]} #{@dimensions[0][:spec]}
+YDEF    #{@dimensions[1][:len]} #{@dimensions[1][:flag]} #{@dimensions[1][:spec]}
+ZDEF    #{@dimensions[2][:len]} #{@dimensions[2][:flag]} #{@dimensions[2][:spec]}
+TDEF    #{@dimensions[3][:len]} #{@dimensions[3][:flag]} #{@dimensions[3][:spec]}
+VARS    #{nvars}
+#{@variables.collect{|i| "  "+i[:name]+" "+i[:nlev].to_s+" "+i[:option].to_s+
+       " "+i[:description]}.join("\n")}
 ENDVARS
 EOS
   end
@@ -436,8 +514,20 @@ ENDVARS
 EOS
   end
 
-  def varnames
-    @variables.collect{|i| i[:name]}
+  def put(ary)
+#    @options["big_endian"] = true
+    ary = ary.hton if ( @options["big_endian"] || @options["cray_32bit_ieee"] )
+    if File.exist?(@dset)
+      print "overwrite "+@dset+" (y/n)? "
+      ans = gets[0].chr
+      if ans != "y"
+        print @dset+" remain unmodified. \n"
+        return
+      end
+    end
+    putfile = File.open(@dset,"w")
+    putfile << ary.to_s
+    putfile.close
   end
 
   def get(name, level, time)
@@ -482,9 +572,10 @@ EOS
 
     ary = NArray.to_na(@datafile.read(@xybytes), "sfloat",
                        @dimensions[0][:len],@dimensions[1][:len])
-    ary = ary.ntoh if ( @options["big_endian"] )
+    ary = ary.ntoh if ( @options["big_endian"] || @options["cray_32bit_ieee"] )
     ary = ary.vtoh if ( @options["little_endian"] )
-    if( @options["big_endian"] && @options["little_endian"] )
+    if( (@options["big_endian"] || @options["cray_32bit_ieee"]) &&
+        @options["little_endian"] )
       raise "endian specification error"
     end
 
@@ -497,10 +588,6 @@ EOS
   attr_accessor(:title, :undef, :dset, :dimensions, :variables)
   attr_reader(:ctlfile)
 
-  def ctlfilename
-    @ctlfile.path
-  end
-
   def get_dim(dim)
     name = dim[:name]
     if (name != 't')
@@ -542,6 +629,10 @@ EOS
     return var_dim
   end
 
+  def varnames
+    @variables.collect{|i| i[:name]}
+  end
+
   def get_alldim 
     ax = Hash.new
     @dimensions.each{|dim|
@@ -551,6 +642,9 @@ EOS
     return ax
   end
 
+  def ctlfilename # obsolete
+    @ctlfile.path
+  end
 
   ########## private methods #############
   private
@@ -901,7 +995,6 @@ class GrADSVar
         @idim = i
       end
     end
-#    p @idim
 
     @attr = Attribute.new
     if( @idim != -1 )
@@ -920,54 +1013,30 @@ class GrADSVar
         end
       end
     end
-#    for i in 0...dimensions.length
-#      dim = dimensions[i]
-#      if @varname == dim[:name]
-#        @dimensions = [ dim ]
-#        @rank = 1
-#      end
-#    end
-#    if ! @dimensions
-#      @dimensions = dimensions
-#      @rank = @dimensions.length
-#    end
-#    p @varname, @dimensions.length, @rank
-#    for i in 0...@ctl.variables.length
-#      if @varname == @ctl.variables[i][:name]
-#        @attr[:long_name] = @ctl.variables[i][:description]
-#        @attr[:nlev] = @ctl.variables[i][:nlev]
-#      end
-#    end
-#    for i in 0...dimensions.length
-#      dim = dimensions[i]
-#      if @varname == dim[:name]
-#        @attr[:name] = dim[:name]
-#        @attr[:long_name] = dim[:description]
-#        @attr[:units] = dim[:units]
-#      end
-#    end
   end
 
   attr_accessor (:rank)
   alias :ndims :rank
 
-  def dim(name)
-  end
+#  def dim(name)
+#  def dims(names)
 
-  def dims(names)
-    
+  def dim_names
+    ary = Array.new()
+    @dimensions.each{|dim| ary.push(dim[:name])}
+    ary
   end
 
-#  def ndims
-#    @rank
-#  end
-
   def name
     @varname
   end
 
   def name=
+    raise "name= not supported"
+  end
 
+  def ndims
+    @rank
   end
 
   def vartype
@@ -979,39 +1048,38 @@ class GrADSVar
     4
   end
 
-  def natts
-  end
-
-
-  def attr
-    @attr
-  end
-
   def att(name)
+    raise "use get_att instead of att"
+  end
+
+  def get_att(name)
     @attr[name]
   end
 
-  def put_att
+  def natts
+    @attr.length
+  end
+
+  def att_names
+    @attr.keys
+  end
+
+  def each_att
+    raise "each_att not supported"
+  end
+
+  def put_att(name,value)
+    @attr[name] = value
+  end
+
+  def attr # obsolete
+    @attr
   end
 
   def file
     @ctl.ctlfile
   end
 
-  def each_att
-    nil
-  end
-
-  def att_names
-
-  end
-
-  def dim_names
-    ary = Array.new()
-    @dimensions.each{|dim| ary.push(dim[:name])}
-    ary
-  end
-
   def shape_ul0
     sh = []
     @dimensions.each{|dim|
@@ -1033,12 +1101,15 @@ class GrADSVar
   end
 
   def scaled_put(var,hash=nil)
+    raise "scaled_put not supported"
   end
 
   def scaled_get(hash=nil)
+    raise "scaled_get not supported"
   end
 
   def put(var,hash=nil)
+    raise "put not supported"
   end
 
   def get(hash=nil)
@@ -1106,51 +1177,6 @@ class GrADSVar
       na
   end
 
-
-#  def get_var(hash=nil)
-##    p hash
-##    na = @ctl.get(@varname,0,0).swap_byte
-#    h_sta = [0, 0, 0, 0]
-#    h_end = [-1, -1, -1, -1]
-#    if hash == nil
-##    elsif hash.key?("index")==true
-#    else
-#      if hash.key?("start")==true
-#        h_sta = hash["start"]
-#        if hash.key?("end")==true
-#          h_end = hash["end"]
-#        end
-#      end
-#      if hash.key?("stride")==true
-#        raise "sorry, stride is not supported yet."
-#      end
-#    end
-#
-#    idx_x = NArray.int(@ctl.dimensions[0][:len]).indgen!
-#    idx_y = NArray.int(@ctl.dimensions[1][:len]).indgen!
-#    idx_z = NArray.int(@ctl.dimensions[2][:len]).indgen!
-#    idx_t = NArray.int(@ctl.dimensions[3][:len]).indgen!
-#    str_x = idx_x[h_sta[0]]; end_x = idx_x[h_end[0]]
-#    str_y = idx_y[h_sta[1]]; end_y = idx_y[h_end[1]]
-#    str_z = idx_z[h_sta[2]]; end_z = idx_z[h_end[2]]
-#    str_t = idx_t[h_sta[3]]; end_t = idx_t[h_end[3]]
-##    p str_x,end_x,str_y,end_y,str_z,end_z,str_t,end_t
-#
-#    for time in str_t..end_t
-#      for lev in str_z..end_z
-##        na_xy = @ctl.get(@varname,lev,time).swap_byte
-#        na_xy = @ctl.get(@varname,lev,time)
-#        if time == str_t && lev == str_z
-#          na = NArray.sfloat(end_x-str_x+1,end_y-str_y+1,end_z-str_z+1,end_t-str_t+1)
-##          p na
-#        end
-#        na[true,true,lev-str_z,time-str_t] = na_xy[str_x..end_x,str_y..end_y]
-#      end
-#    end
-#    p na
-#    na
-#  end
-
   alias simple_put put
   alias simple_get get
 
@@ -1248,7 +1274,7 @@ if __FILE__ == $0 then
   end
   print gr.to_ctl
   p gr
-  p gr.ctlfilename
+  p gr.path
   p gr.var_names
   p gr.get_dim(gr.dimensions[2])
   p gr.get('T',0,0)
