@@ -441,6 +441,9 @@ These methods returns a NArray (not a VArray).
 	attr[name]=val
 	self
       end
+      def del_att(name)
+	attr.delete(name)
+      end
       alias put_att set_att
 
       def units
@@ -486,6 +489,10 @@ These methods returns a NArray (not a VArray).
 	 end
       end
 
+      def transpose(*dims)
+	VArray.new( val.transpose(*dims), @attr.copy, name )
+      end
+
       ### < NArray methods > ###
 
       ## ToDo: implement units handling
@@ -493,27 +500,35 @@ These methods returns a NArray (not a VArray).
 
       def coerce(other)
 	oattr = self.attr.copy
-	oattr['units'] = '1'    # Assume the other to be non-dimensional
 	case other
-	when Numeric
-	  na_other, = NArray.new(self.typecode, 1).coerce(other)   # scalar
+	when UNumeric
+	  oattr['units'] = other.units.to_s
+	  na_other, = NArray.new(self.typecode, 1).coerce(other.val)   # scalar
 	  c_other = VArray.new(na_other, oattr, self.name)
-	when Array
-	  na = NArray.to_na(other)
-	  c_other = VArray.new(na, oattr, self.name)
-	when NArray
-	  c_other = VArray.new(other, oattr, self.name)
 	else
-	  raise "Cannot coerse #{other.class}"
+	  oattr['units'] = '1'    # Assume the other to be non-dimensional
+	  case other
+	  when Numeric
+	    na_other, = NArray.new(self.typecode, 1).coerce(other)   # scalar
+	    c_other = VArray.new(na_other, oattr, self.name)
+	  when Array
+	    na = NArray.to_na(other)
+	    c_other = VArray.new(na, oattr, self.name)
+	  when NArray
+	    c_other = VArray.new(other, oattr, self.name)
+	  else
+	    raise "Cannot coerse #{other.class}"
+	  end
 	end
 	[c_other, self]
       end
 
       Math_funcs_nondim = ["exp","log","log10","log2","sin","cos","tan",
-	    "sinh","cosh","tanh","asin","acos","atan","atan2","asinh","acosh",
-	    "atanh","csc","sec","cot","csch","sech","coth","acsc","asec",
-	    "acot","acsch","asech","acoth"]
-      Math_funcs = Math_funcs_nondim + ["sqrt"]
+	    "sinh","cosh","tanh","asinh","acosh",
+	    "atanh","csc","sec","cot","csch","sech","coth",
+	    "acsch","asech","acoth"]
+      Math_funcs_radian = ["asin","acos","atan","atan2","acsc","asec","acot"]
+      Math_funcs = Math_funcs_nondim + Math_funcs_radian + ["sqrt"]
 
       Binary_operators_Uop   = ["*","/","**", ".mul!",".div!"]
       Binary_operators_Uconv = ["+","-",".add!",".sub!"]
@@ -550,17 +565,30 @@ These methods returns a NArray (not a VArray).
 
       for f in Math_funcs_nondim
 	 eval <<-EOS, nil, __FILE__, __LINE__+1
-	 #def VArray.#{f}(vary)
-         #   raise ArgumentError, "Not a VArray" if !vary.is_a?(VArray)
-	 #   VArray.new( Misc::EMath.#{f}(vary.val), vary.attr.copy, vary.name )
-	 #end
 	 def #{f}(*arg)
             newattr = self.attr.copy
             newattr['units'] = '1'
 	    case arg.length
 	    when 0
 	      VArray.new( Misc::EMath.#{f}(self.val), newattr, self.name )
-	    when 1
+	    #when 1  # for atan2
+	    #  ar = arg[0].respond_to?(:val) ? arg[0].val : arg[0]
+	    #  VArray.new( Misc::EMath.#{f}(self.val, ar), newattr, self.name )
+	    else
+	      raise ArgumentError, "# of args must be 0 or 1"
+	    end
+	 end
+         EOS
+      end
+      for f in Math_funcs_radian
+	 eval <<-EOS, nil, __FILE__, __LINE__+1
+	 def #{f}(*arg)
+            newattr = self.attr.copy
+            newattr['units'] = 'rad'
+	    case arg.length
+	    when 0
+	      VArray.new( Misc::EMath.#{f}(self.val), newattr, self.name )
+	    when 1  # for atan2
 	      ar = arg[0].respond_to?(:val) ? arg[0].val : arg[0]
 	      ## ar = ar.to_f   # NMath.atan2 does not work with NArray.int
 	      VArray.new( Misc::EMath.#{f}(self.val, ar), newattr, self.name )
@@ -783,6 +811,8 @@ if $0 == __FILE__
    p "@@@",vs.rank,vs.shape,vs.total,vs.val,vs.get_att("name")
    vs.val=999
    p "*1*",va
+   co,=va.coerce(UNumeric.new(1,Units.new("rad")))
+   p '*coerce*',co, co.units
    p "*2*",vt = vs/9, vs + vt
    p "*3*",vt.log10
    p "*4*",(vt < vs)
